import { Component, h, Prop, Watch, Event } from '@stencil/core';
import IframeMessenger from '../../services/iframeMessenger';
import IframeSession from '../../services/iframeSession';
export class QuickomChatIframe {
  constructor() {
    this.domain = 'https://quickom-demo.web.app';
    this.chatAccount = null;
    this.chatWithAccount = null;
    this.iframeElement = null;
    this.iframeMessenger = null;
    this.iframeSession = null;
  }
  getUrl() {
    return `${this.domain}/chat`;
  }
  onIframeRef(el) {
    this.iframeElement = el;
  }
  onLoadIframe() {
    this.iframeMessenger = new IframeMessenger(this.iframeElement);
    this.iframeMessenger.start();
    this.iframeSession = IframeSession.init(this.iframeMessenger);
    this.sendUpdateChatSettings();
    this.iframeSession.on('message', this.onChatBoxMessage.bind(this));
  }
  sendUpdateChatSettings() {
    if (this.iframeSession && this.chatAccount && this.chatWithAccount) {
      this.iframeSession.send({
        resource: 'chat',
        method: 'updateChatSettings',
        params: [{ chatAccount: this.chatAccount, chatWithAccount: this.chatWithAccount }],
      });
    }
  }
  onChatBoxMessage(data) {
    switch (data.method) {
      case 'show':
        this.visible.emit(true);
        break;
      case 'hide':
        this.visible.emit(false);
        break;
    }
  }
  render() {
    return h("iframe", { onLoad: () => this.onLoadIframe(), ref: el => this.onIframeRef(el), src: this.getUrl(), width: "100%", height: "100%", frameborder: "0" });
  }
  onChangeChatAccount() {
    this.sendUpdateChatSettings();
  }
  onChangeChatWithAccount() {
    this.sendUpdateChatSettings();
  }
  disconnectedCallback() {
    this.iframeSession.destroy();
    this.iframeMessenger.stop();
    this.iframeMessenger = null;
    this.iframeSession = null;
  }
  static get is() { return "quickom-chat-iframe"; }
  static get originalStyleUrls() { return {
    "$": ["quickom-chat-iframe.css"]
  }; }
  static get styleUrls() { return {
    "$": ["quickom-chat-iframe.css"]
  }; }
  static get properties() { return {
    "domain": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "domain",
      "reflect": false,
      "defaultValue": "'https://quickom-demo.web.app'"
    },
    "chatAccount": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "ChatAccount",
        "resolved": "ChatAccount",
        "references": {
          "ChatAccount": {
            "location": "import",
            "path": "../../dtos/chatAccount"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "defaultValue": "null"
    },
    "chatWithAccount": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "ChatWithAccount",
        "resolved": "ChatWithAccount",
        "references": {
          "ChatWithAccount": {
            "location": "import",
            "path": "../../dtos/chatWithAccount"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "defaultValue": "null"
    }
  }; }
  static get events() { return [{
      "method": "visible",
      "name": "visible",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": ""
      },
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      }
    }]; }
  static get watchers() { return [{
      "propName": "chatAccount",
      "methodName": "onChangeChatAccount"
    }, {
      "propName": "chatWithAccount",
      "methodName": "onChangeChatWithAccount"
    }]; }
}
